#! /usr/bin/env node

/*eslint-env node*/
/*eslint no-console: 0*/

var fs = require('fs'),
  rollup = require('rollup'),
  uglify = require('uglify-js'),
  zlib = require('zlib'),
  outputs = [
    {
      rollupOptions:{
        format: 'iife',
        moduleName: 'j2cPrefixPluginBrowser'
      },
      name: 'global',
      minify: {saveGzip:true}
    },
    {
      rollupOptions:{
        format: 'cjs'
      },
      name: 'commonjs',
      minify: {save:true}
    },
    {
      rollupOptions:{
        format: 'amd',
        name: 'j2c-prefix-plugin-browser'
      },
      name: 'amd',
      minify: {save:true}
    }
  ]

var parsed = rollup.rollup({
  entry: 'src/main.js'
})


outputs.forEach(function (output) {
  parsed.then(function(bundle){
    var result = bundle.generate(output.rollupOptions)
    fs.writeFileSync('dist/j2c-plugin-prefix-browser.' + output.name + '.js', result.code)
    if (output.minify) {
      var minified = uglify.minify(result.code, {
        fromString: true,
        mangle: true,
        compress: {}
      }).code
      fs.writeFileSync('dist/j2c-plugin-prefix-browser.' + output.name + '.min.js', minified)
      zlib.gzip(minified, function(_, buf){
        console.log(output.name, _ || buf.length)
        if (output.minify.saveGzip) {
          fs.writeFileSync('dist/j2c-plugin-prefix-browser.' + output.name + '.min.js.gz', buf)
        }
      })
    }
  }).catch(function (e) {
    console.log(output.name, e)
    process.exit(1)
  })
})

rollup.rollup({
  entry: 'test-utils/_exposed.es6.js'
}).then(
  function(bundle){
    var result = bundle.generate({format:'cjs'})
    fs.writeFileSync(
      'test-utils/exposed.js',
      '// This file is generated by `../script/build.js`, from `./_exposed.es6.js`\n' + result.code
    )
  }
).catch(function(err) {
  console.error(err)
  process.exit(1)
})